# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import unittest
import mock
from . import api
from .compat import urlparse

class UtilsTest(unittest.TestCase):
    def test_binary_encode(self):
        self.assertEqual(api.encode_to_binary('привет'), b'043f04400438043204350442')

    def test_phone_normalize(self):
        self.assertEqual(api.normalize_phone('79991234567'), '79991234567')
        self.assertEqual(api.normalize_phone('+7 (999) 1234567'), '79991234567')
        self.assertEqual(api.normalize_phone('8(999)1234-567'), '79991234567')
        self.assertEqual(api.normalize_phone('89991234567'), '79991234567')


@mock.patch('imobis.api.urlopen')
class ApiTest(unittest.TestCase):

    def setUp(self):
        self.im = api.Imobis('login', 'password')

    def assertUrlQueryEqual(self, url1, url2):
        data1 = urlparse.urlsplit(url1)
        data2 = urlparse.urlsplit(url2)
        q1 = urlparse.parse_qs(data1.query)
        q2 = urlparse.parse_qs(data2.query)
        self.assertEqual(q1, q2)


    def test_send_sms(self, urlopen):
        urlopen.return_value.read.return_value = b'124'
        url = b'http://gate.sms-manager.ru/_getsmsd.php?user=login&password=password&sender=Pan%20Gurman&binary=043f04400438043204350442&GSM=79991234567&messageId=5'

        res = self.im.send_sms('Pan Gurman', '8 (999) 1234-567', 'привет', 5)

        request_url = urlopen.call_args[0][0]
        self.assertUrlQueryEqual(request_url, url)

        self.assertEqual(res, 124)

    def test_sms_send_str(self, urlopen):
        urlopen.return_value.read.return_value = b'124'
        url = b'http://gate.sms-manager.ru/_getsmsd.php?user=login&password=password&sender=Pan%20Gurman&binary=043f04400438043204350442&GSM=79991234567'

        im = api.Imobis(b'login', b'password')
        res = im.send_sms(b'Pan Gurman', '8 (999) 1234-567', 'привет')

        request_url = urlopen.call_args[0][0]
        self.assertUrlQueryEqual(request_url, url)

        self.assertEqual(res, 124)

    def test_errors(self, urlopen):
        urlopen.return_value.read.return_value = b'-1'

        raised = False
        try:
            self.im.send_sms('Pan Gurman', '8 (999) 1234-567', 'привет')
        except api.ImobisError as e:
            self.assertEqual(e._code, -1)
            self.assertEqual(e.message(), 'Ошибка отправки')
            raised = True
        self.assertTrue(raised)

    def test_balance(self, urlopen):
        urlopen.return_value.read.return_value = b'100'
        self.assertEqual(self.im.get_balance(), 100.0)

    def test_is_valid_phone(self, urlopen):
        urlopen.return_value.read.return_value = b'OK'
        self.assertTrue(self.im.is_valid_phone('1234234234'))

    def test_is_valid_phone_error_format(self, urlopen):
        urlopen.return_value.read.return_value = b'-9'
        self.assertFalse(self.im.is_valid_phone('1234234234'))

    def test_is_valid_phone_error(self, urlopen):
        urlopen.return_value.read.return_value = b'-10'

        def fetch():
            return self.im.is_valid_phone('123423412')

        self.assertRaises(api.ImobisError, fetch)




# -*- coding: utf-8 -*-
from __future__ import absolute_import

try:
    from urllib.request import urlopen
    import urllib.parse as urlparse
    from urllib.parse import urlencode
except ImportError:
    import urlparse
    from urllib2 import urlopen
    from urllib import urlencode



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import re
import binascii
from xml.etree import ElementTree
from .compat import urlopen, urlencode

REMOVE_RE = re.compile(r'[\s\(\)\-]', re.U)

class ImobisError(Exception):
    ERRORS = {
        -1: 'Ошибка отправки',
        -2: 'Не достаточно средств на балансе для отправки сообщения',
        -3: 'Неизвестный номер',
        -4: 'Внутренняя ошибка',
        -5: 'Неверный логин или пароль',
        -6: 'Отсутствует номер получателя',
        -7: 'Отсутствует текст сообщения',
        -8: 'Отсутствует имя отправителя',
        -9: 'Неверный формат номера получателя',
        -10: 'Отсутствует логин',
        -11: 'Отсутствует пароль',
        -12: 'Неверный формат внешнего (external) Id',
    }

    def __init__(self, code):
        self._code = code

    def __str__(self):
         return "ImobisError %s" % self._code

    def message(self):
        return self.ERRORS.get(self._code, 'unknown error %s' % self._code)


def encode_to_binary(text):
    return binascii.hexlify(text.encode('UTF-16BE'))

def normalize_phone(phone):
    phone = re.sub(REMOVE_RE, '', phone).replace('+7', '7')
    if phone.startswith('8'):
        phone = '7'+phone[1:]
    return phone


class Imobis(object):
    GATE_URL = 'http://gate.sms-manager.ru/_getsmsd.php'
    BALANCE_URL = 'http://gate.sms-manager.ru/_balance.php'
    CHECK_URL = 'http://gate.sms-manager.ru/_checkgsm.php'

    def __init__(self, user, password, timeout=10):
        self.user = user
        self.password = password
        self.timeout = timeout

    def send_sms(self, sender, phone, message, message_id=None):
        """
        Sends SMS. Returns internal_id.
        """
        data = {
            'sender': sender,
            'GSM': normalize_phone(phone),
            'binary': encode_to_binary(message)
        }
        if message_id is not None:
            data['messageId'] = message_id

        result = int(self._http_get(self.GATE_URL, data))

        if result < 0:
            raise ImobisError(result)

        return result

    def get_balance(self):
        """
        Returns current balance.
        """
        return float(self._http_get(self.BALANCE_URL, {}).decode('utf8'))

    def is_valid_phone(self, phone):
        """
        Checks if phone is valid and returns True or False.
        """
        res = self._http_get(self.CHECK_URL, {
            'GSM': normalize_phone(phone),
            'mode': 'brief',
        }).decode('utf8')

        if res == 'OK':
            return True
        if res == 'noBindingDetected':
            return False

        try:
            code = int(res)
            if code == -9: # invalid phone format
                return False
            raise ImobisError(code)
        except ValueError:
            raise ImobisError(res)

# TODO:
#    def get_phone_info(self, phone):
#        """
#        Returns phone info: dict(
#            region='Санкт-Петербург',
#            operator='ОАО "Мобильные Телесистемы"',
#            issuedate='2001-10-18'
#        )
#
#        or None if phone is invalid.
#        """
#
#        res = self._http_get(self.CHECK_URL, {
#            'GSM': normalize_phone(phone),
#            'mode': 'full',
#        }).decode('utf8')
#
#        xml = ElementTree.fromstring(res)
#        return xml


    def _http_get(self, url, data):
        _data = {'user': self.user, 'password': self.password}
        _data.update(data)
        url = url + '?' + urlencode(_data)
        return urlopen(url.encode('utf8'), timeout=self.timeout).read()



#!/usr/bin/env python
from distutils.core import setup

for cmd in ('egg_info', 'develop'):
    import sys
    if cmd in sys.argv:
        from setuptools import setup

try:
    import sys
    reload(sys).setdefaultencoding("UTF-8")
except NameError:
    # python3 doesn't need this
    pass

version='0.3'

setup(
    name='imobis',
    version=version,
    author='Mikhail Korobov',
    author_email='kmike84@gmail.com',

    packages=['imobis'],

    url='https://bitbucket.org/kmike/imobis/',
    license = 'MIT license',
    description = "Python interface to http://sms-manager.ru/ (aka http://www.imobis.ru/ )",

    long_description = open('README.rst').read(),

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
